# -*- coding: utf-8 -*-
"""pysourceinfo.helper - miscellaneous utilities.

Details see @local-manuals or @[https://pythonhosted.org/pysourceinfo/]
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import re
from inspect import stack
from imp import PY_SOURCE, PY_COMPILED, C_EXTENSION, PKG_DIRECTORY, C_BUILTIN, PY_FROZEN

from pysourceinfo import V3K
from pysourceinfo import PySourceInfoError
from pysourceinfo import presolve, P_FIRST, P_LAST, P_LONGEST, P_SHORTEST, P_IGNORE0

if V3K:
    # pylint: disable-msg=F0401
    from importlib.machinery import SOURCE_SUFFIXES, DEBUG_BYTECODE_SUFFIXES, OPTIMIZED_BYTECODE_SUFFIXES, BYTECODE_SUFFIXES, EXTENSION_SUFFIXES  # @UnresolvedImport
    from importlib import util  # @UnresolvedImport
    # pylint: enable-msg=F0401
else:
    from inspect import getmoduleinfo  # @UnresolvedImport


__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
    " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"


# redundant for dependency reduction - keep in sync with pysourceinfo.objectinfo
__MT_UNKNOWN = 0
__MT_SOURCE = 1  # PY_SOURCE #1
__MT_COMPILED = 2  # PY_COMPILED #2
__MT_EXTENSION = 3  # C_EXTENSION #3
__MT_DIRECTORY = 5  # PKG_DIRECTORY #5
__MT_BUILTIN = 6  # C_BUILTIN #6
__MT_FROZEN = 7  # PY_FROZEN #7
__MT_COMPILED_OPT1 = 10  # PY_COMPILED | <opt1> # 2 | 8
__MT_COMPILED_OPT2 = 18  # PY_COMPILED | <opt2> # 2 | 16
__MT_COMPILED_DEBUG = 34  # PY_COMPILED # 2


_scname = re.compile(
    r'''.*[\\\\/]([^.]+)[.]([^-]+)-([0-9]{2})[.](opt-[012])*([.].+)$''')


def getfilepathname_type(fpath):
    """Type for stored module as defined by *imp*."""

    if not V3K:  # for now the more frequent call
        if not os.path.exists(fpath):
            return
        ret = getmoduleinfo(fpath)
        if ret and ret[3] in (PY_SOURCE, PY_COMPILED, C_EXTENSION, PKG_DIRECTORY, C_BUILTIN, PY_FROZEN):
            return ret[3]

    else:
        if not os.path.exists(fpath):
            return

        #
        # the redundnacy of type-postfixes requires now prefix-analysis by '_scname'
        #
        
        # In [1]: from importlib.machinery import SOURCE_SUFFIXES, DEBUG_BYTECODE_SUFFIXES
        #    ...: , OPTIMIZED_BYTECODE_SUFFIXES, BYTECODE_SUFFIXES, EXTENSION_SUFFIXES
        # 
        # In [3]: SOURCE_SUFFIXES
        # Out[3]: ['.py']
        # 
        # In [4]: DEBUG_BYTECODE_SUFFIXES
        # Out[4]: ['.pyc']
        # 
        # In [5]: OPTIMIZED_BYTECODE_SUFFIXES
        # Out[5]: ['.pyc']
        # 
        # In [6]: BYTECODE_SUFFIXES
        # Out[6]: ['.pyc']
        # 
        # In [7]: EXTENSION_SUFFIXES
        # Out[7]: ['.cpython-36m-x86_64-linux-gnu.so', '.abi3.so', '.so']

        x = _scname.findall(fpath)
        if x:
            x = x[0]  # trust...
        else:
            x = os.path.splitext(fpath)

        if x[-1] in SOURCE_SUFFIXES:
            return __MT_SOURCE
        elif x[-1] in EXTENSION_SUFFIXES:
            return __MT_EXTENSION
        elif x[-1] in BYTECODE_SUFFIXES and not x[-2]:
            return __MT_COMPILED
        elif x[-1] in OPTIMIZED_BYTECODE_SUFFIXES and x[-2]:
            if x[-2] == 'opt-1':
                return __MT_COMPILED_OPT1
            if x[-2] == 'opt-2':
                return __MT_COMPILED_OPT2
            raise PySourceInfoError(
                "Unknown opt:" + str(x[-2]) + " " + str(fpath))
        elif x[-1] in DEBUG_BYTECODE_SUFFIXES:
            return __MT_COMPILED_DEBUG
    return __MT_UNKNOWN


def get_oid_filepathname(oid, spath=None):
    """File path name for OID."""
    find_spec = util.find_spec(oid, spath)
    if not find_spec is None:
        return find_spec.origin


def getpythonpath(pname, plist=None, **kw):
    """Matches prefix from sys.path."""
    if not plist:
        plist = sys.path
    _fp = os.path.normpath(os.path.abspath(pname))
    _presolve = kw.get('presolve', presolve)
    _buf = ''
    for _sp in plist:
        _sp = os.path.normpath(os.path.abspath(_sp))
        if _fp and _fp.startswith(_sp):
            if _presolve == P_FIRST:
                return _sp
            elif _presolve == P_LAST:
                _buf = _sp
            elif _presolve == P_LONGEST:
                if len(_sp) > len(_buf):
                    _buf = _sp
            elif _presolve == P_SHORTEST:
                if len(_sp) < len(_buf) or not _buf:
                    _buf = _sp
    return _buf


def getpythonpath_rel(fpname, plist=None, **kw):
    """Relative path name for first match on plist."""
    _p = getpythonpath(fpname, plist, **kw)
    if not _p:
        return
    ret = os.path.normpath(fpname)[len(_p):]
    if ret and ret[0] == os.path.sep:
        return ret[1:]
    return ret


def getstack_frame(spos=1, fromtop=False):
    """List of current mem-addresses on stack frames."""
    if fromtop:
        return stack()[-1 * spos]
    return stack()[spos]


def getstack_len():
    """Length of current stack."""
    return len(stack())

def matchpath(path, pathlist=None, pmatch=P_FIRST):
    """Match a file pathname of pathname on a pathlist."""
    _buf = None
    if pathlist == None:
        pathlist = sys.path
        
    if pmatch & P_IGNORE0:
        _pl = pathlist[1:]
    else:
        _pl = pathlist
    for sx in _pl:
        p0 = os.path.normpath(sx)
        if path.startswith(p0):
            if pmatch & P_FIRST:
                return p0
            elif pmatch & P_LAST:
                _buf = p0
            elif pmatch & P_LONGEST:
                if len(p0) > len(_buf):
                    _buf = p0
            elif pmatch & P_SHORTEST:
                if len(p0) < len(_buf) or not _buf:
                    _buf = p0
    return _buf




# -*- coding: utf-8 -*-
"""pysourceinfo.fileinfo - information on source files.

Based on the stack-frames of *inspect*, *__file__*, and additional 
attributes.

Details see @local-manuals or @[https://pythonhosted.org/pysourceinfo/]
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import re

from inspect import stack, getmodule, isbuiltin, ismodule, getsourcefile

from pysourceinfo import V3K
from pysourceinfo import presolve
from pysourceinfo.helper import matchpath, getpythonpath, getpythonpath_rel

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
    " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"


if V3K:
    vers = '{0}{1}'.format(*sys.version_info[:2])   # pylint: disable-msg=W0142


#---
#
# redundant for autonomous load-independence by reduction of dependencies
#
__gmn = re.compile(r'^.*[.]')
def __getmodule_name(mod):
    #"""Name similar to basename of loaded module, same as getmodule_oid."""
    if mod and ismodule(mod):
        if hasattr(mod, '__name__'):
            return __gmn.sub('', mod.__name__)
#---

def getcaller_filename(spos=1):
    """Filename of caller's module."""
    return os.path.basename(getcaller_filepathname(spos))


def getcaller_filepathname(spos=1):
    """File pathname of caller's module."""
    _sf = stack()
    return os.path.abspath(_sf[spos][1])


def getcaller_filepathname_rel(spos=1):
    """File pathname of caller's module relative to sys.path."""
    _sf = stack()
    _cf = os.path.abspath(os.path.normpath(_sf[spos][1]))
    for sx in sys.path:
        if _cf.startswith(sx):
            return _cf[len(os.path.normpath(sx))]


def getcaller_linenumber(spos=1):
    """Source line number of call."""
    return stack()[spos][2]


def getcaller_linenumber_def(spos=1):
    """First line number of enclosing caller function/method definition."""
    return stack()[spos][0].f_code.co_firstlineno

def getcaller_package_filename(spos=1):
    return os.path.basename(getcaller_package_filepathname(spos))


__comp_package_filepathname = re.compile(r"""
    (
        ([/\\\\]*([^/\\\\]+.py)[oc]{0,1}$)
        | ([/\\\\]*([^/\\\\]+).*.py[oc]{0,1}$)
        | ([/\\\\]*([^/\\\\]+).*$)
    )
    """, re.VERBOSE
)
def getcaller_package_filepathname(spos=1, pmatch=presolve):
    """Filepathname of the package from *sys.path*."""
    _sf = stack()
    _cf = os.path.normpath(os.path.abspath(_sf[spos][1]))

    p0 = matchpath(_cf, sys.path, pmatch)

    p0l = len(p0)
    p1 = getcaller_filepathname(spos + 1)[p0l:]
    pfn = __comp_package_filepathname.search(p1)

    # TODO: PEP420 - Implicit Namespace Packages

    if pfn.start(2) != -1:
        return p0 + os.path.sep + pfn.group(3)
    elif pfn.start(4) != -1:
        return p0 + os.path.sep + pfn.group(5)
    elif pfn.start(6) != -1:
        return p0 + os.path.sep + pfn.group(7)
    return None


def getcaller_package_pathname(spos=1, pmatch=presolve):
    """Pathname of the package from *sys.path* as used for search of the package."""
    return os.path.dirname(getcaller_package_filepathname(spos+1, pmatch))


def getcaller_pathname(spos=1):
    """pathname of caller source file."""
    if not V3K:
        return os.path.dirname(getcaller_filepathname(spos + 1))
    else:
        _sf = stack()
        # mx = _sf[spos][0].f_locals['module']
        module = getmodule(_sf[spos][0])
        if hasattr(module, '__file__'):
            f = module.__file__
            return os.path.dirname(f)
        # PEP 3147 -- PYC Repository Directories


__comp_getcaller_pathname_rel = re.compile(r"""
    (
     (()[/\\\\]*[^/\\\\]+.py[oc]{0,1}$)
     |([/\\\\]*(.+)[/\\\\][^/\\\\]+.py[oc]{0,1}$)
     |([/\\\\]*(.+))
    )
    """, re.VERBOSE
)


def getcaller_pathname_rel(spos=1):
    """Relative pathname to first matching package directory of caller."""
    ax = os.path.normpath(getcaller_filepathname(
        spos + 1))  # TODO: check for normpath
    for si in sys.path:
        si = os.path.normpath(si) 
        if ax.startswith(si):
            gx = __comp_getcaller_pathname_rel.search(ax[len(si):])
            if gx.start(3) != -1:
                return gx.group(3)
            elif gx.start(5) != -1:
                return gx.group(5)
            elif gx.start(7) != -1:
                return gx.group(7)
            return None


def getcaller_pathname_sub(spos=1):
    """sub-pathname to first matching module directory of caller."""
    return re.sub(r"[^/\\\\]*[/\\\\](.*)", r"\1", getcaller_pathname_rel(spos + 1))


def getcaller_python_pathname(spos=1):
    """Alias for getcaller_package_pathname."""
    return getcaller_package_pathname(spos + 1)


def getcaller_source_filepathname(spos=1):
    """Pathname of caller source."""
    return os.path.abspath(getsourcefile(stack()[spos][0]))  # mx = _sf[spos][1]


def getmodule_filename(mod):
    """Basename of file for loaded module *mod*."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__') and os.path.exists(mod.__file__):
            return os.path.basename(mod.__file__)


def getmodule_filepathname(mod):
    """File pathname of loaded module *mod*."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        return os.path.abspath(getsourcefile(mod))


def getmodule_package_pathname(mod):
    """Path name of package for loaded module."""
    mn = __getmodule_name(mod)  # requires normalized path
    if mn:
        mpn = getmodule_pathname(mod)
        return mpn[:len(mpn) + 1]


def getmodule_pathname(mod):
    """Path name of loaded module."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__'):
            return os.path.normpath(os.path.dirname(mod.__file__))


def getmodule_pathname_rel(mod, plist=None):
    """Relative path name to PYTHONPATH for loaded module."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__'):
            return getpythonpath_rel(mod.__file__, plist)


def getmodule_pathname_sub(mod, plist=None):
    """Path name for loaded module, relative to package path ."""
    ax = getmodule_pathname_rel(mod, plist)
    if not V3K:
        return re.sub(r"[^/\\\\]*[/\\\\]", r"\1", ax)
    else:
        return re.sub(r"[^/\\\\]*[/\\\\]".encode('utf-8'), r"\1".encode('utf-8'), ax)


def getmodule_python_pathname(mod, plist=None):
    """Path name from PYTHONPATH of loaded module."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__'):
            return getpythonpath(mod.__file__, plist)




# -*- coding: utf-8 -*-
"""'pysourceinfo' - Python RTTI based on 'inspect'"""
from __future__ import absolute_import
from sys import version_info

__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
                " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = 'efed42d3-f801-4fbb-abfd-bd598d683a82'
__docformat__ = "restructuredtext en"


class PySourceInfoError(Exception):
    """Common parent exception for the *pysourceinfo* package."""
    def __init__(self, *args, **kw):
        super(PySourceInfoError, self).__init__(*args, **kw)

V3K = False  # : Python3.5+
if version_info[:2] > (3, 4,):
    V3K = True
    _STR = (str,)  # : string and unicode
    from importlib import find_loader  # @UnresolvedImport
    vers = '{0}{1}'.format(*version_info[:2])  # pylint: disable-msg=W0142
elif version_info[:2] > (2, 6,) and version_info[:2][0] < 3:
    _STR = (str, unicode,)  # : string and unicode @UndefinedVariable
else:
    raise PySourceInfoError(
        "Requires Python 2.7+, or 3.5+:" + str(version_info[:2]))


# deprecated with 3.3/__pycache__, but still require these, going to use own values than
# re-map for smart import/export
MT_UNKNOWN = 0
MT_SOURCE = 1  #: same value as PY_SOURCE = 1
MT_COMPILED = 2  #: same value as PY_COMPILED = 2
MT_EXTENSION = 3  # : same value as C_EXTENSION = 3
MT_DIRECTORY = 5  # : same value as PKG_DIRECTORY = 5
MT_BUILTIN = 6  # : same value as C_BUILTIN = 6
MT_FROZEN = 7  # : same value as PY_FROZEN = 7

MT_COMPILED_OPT = 10  # : PY_COMPILED | <opt1 or opt2> # 2 | 8
MT_COMPILED_OPT1 = 18  # : PY_COMPILED | <opt1> # 2 | 16
MT_COMPILED_OPT2 = 34  # : PY_COMPILED | <opt2> # 2 | 32
MT_COMPILED_DEBUG = 66  # : PY_COMPILED | 0 # 2 | 64


#
# bits defining the evaluated package path 
# from PYTHONPATH in case of multiple matches
#
P_FIRST = 1  #: first matched path-prefix
P_LAST = 2  #: last matched path-prefix
P_SHORTEST= 4  #: longest matched path-prefix
P_LONGEST = 8  #: longest matched path-prefix
P_IGNORE0 = 16  #: ignores sys.path[0] - the execution path

presolve = P_FIRST  #: default value for sys.path resolution

__all__ = [
    'bininfo',
    "fileinfo.py",
    "helper.py",
    "infolists.py",
    "objectinfo.py",
]



# -*- coding: utf-8 -*-
"""pysourceinfo.infolists - lists.

Details see @local-manuals or @[https://pythonhosted.org/pysourceinfo/]
"""
from __future__ import absolute_import

import os
import sys
import re
from itertools import groupby

from pysourceinfo.helper import getpythonpath, getpythonpath_rel

# if V3K:
    # from importlib.machinery import SOURCE_SUFFIXES,DEBUG_BYTECODE_SUFFIXES,
    #    OPTIMIZED_BYTECODE_SUFFIXES,BYTECODE_SUFFIXES,
    #    EXTENSION_SUFFIXES #@UnresolvedImport
    # from importlib import util #@UnresolvedImport


__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
    " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"


#---
#
# redundant for autonomous load-independence by reduction of dependencies
#


def __sortu(sequence):
    """sort unique"""
    return (x[0] for x in groupby(sorted(sequence)))


#---

def getsysmodules_filepathname_list(r=None):
    """Returns the list of file pathnames of all loaded modules."""
    if not r:
        return sorted([x.__file__ for x in sys.modules.values()
                       if x and hasattr(x, '__file__')])
    return sorted([x.__file__ for x in sys.modules.values()
                   if x and hasattr(x, '__file__') and re.search(re.escape(r), x.__file__)])


def getsysmodules_id_list(r=None):
    """Returns the list of the IDs of the loaded modules."""
    if not r:
        return sorted([id(x) for x in sys.modules.values() if x])
    return sorted([id(x) for x in sys.modules.values()
                   if x and re.search(str(r), str(id(x)))])


def getsysmodules_list(r=None):
    """Returns the list of the loaded modules."""
    if not r:
        ml = sorted([x for x in sys.modules.keys()
                     if sys.modules[x]])
        return map(lambda x: sys.modules[x], ml)
    ml = sorted([x for x in sys.modules.keys()
                 if sys.modules[x] and re.search(r, x)])
    return map(lambda x: sys.modules[x], ml)


def getsysmodules_name_list(r=None):
    """Returns the list of the loaded modules."""
    if not r:
        return sorted([x for x in sys.modules.keys()
                       if sys.modules[x]])
    return sorted([x for x in sys.modules.keys()
                   if sys.modules[x] and re.search(re.escape(r), x)])


def getsysmodules_pathname_list(r=None):
    """Returns the list of path names of the loaded modules."""
    if not r:
        return list(__sortu([os.path.normpath(os.path.dirname(x.__file__))
                             for x in sys.modules.values() if x and
                             hasattr(x, '__file__')]))
    return list(__sortu([os.path.normpath(os.path.dirname(x.__file__))
                         for x in sys.modules.values()
                         if x and hasattr(x, '__file__') and
                         re.search(re.escape(r), x.__file__)]))


def getsysmodules_pathname_rel_list(r=None, plist=None):
    """Returns for the loaded modules the list of
    path names relative to PYTHONPATH.
    """
    if not r:
        return list(__sortu([getpythonpath_rel(x.__file__, plist)
                             for x in sys.modules.values()
                             if x and hasattr(x, '__file__')]))
    return list(__sortu([getpythonpath_rel(x.__file__, plist)
                         for x in sys.modules.values()
                         if x and hasattr(x, '__file__') and
                         re.search(re.escape(r), x.__file__)]))


def getsysmodules_python_pathname_list(r=None, plist=None):
    """Returns the list of path names for the loaded modules
    from PYTHONPATH."""
    if not r:
        return list(__sortu([getpythonpath(x.__file__, plist)
                             for x in sys.modules.values()
                             if x and hasattr(x, '__file__')]))
    return list(__sortu([getpythonpath(x.__file__, plist)
                         for x in sys.modules.values()
                         if x and hasattr(x, '__file__') and
                         re.search(re.escape(r), x.__file__)]))





# -*- coding: utf-8 -*-
"""pysourceinfo.bininfo - runtime information on compiled Python binaries.

Details see @local-manuals or @[https://pythonhosted.org/pysourceinfo/]
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import re
from inspect import stack, getmodule, isbuiltin, ismodule

from pysourceinfo import V3K


__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
                " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"

#---
#
# redundant for autonomous load-independence by reduction of dependencies
#
def __getpythonpath_rel(fpname, plist=None):
    """Relative path name for first match on plist."""
    if not plist:
        plist = sys.path
    # for now dirs with terminating os.sep
    _fp = os.path.normpath(os.path.abspath(fpname))
    _fp = re.escape(_fp)
    for _sp in plist:
        _sp = os.path.normpath(os.path.abspath(_sp))
        _sp = re.escape(_sp)
        if _fp and _fp.startswith(_sp):
            _r = _fp.replace(_sp, "")
            if _r and _r[0:2] == re.escape(os.sep):
                return re.sub(r'\\(.)', r'\1', _r[2:])
            elif _r and _r[0] == os.sep:
                return re.sub(r'\\(.)', r'\1', _r[1:])
            if not _r:
                return '.'
            return re.sub(r'\\(.)', r'\1', _r)
#---


def getcaller_bin_filename(spos=1):
    """Filename of binary caller module."""
    return os.path.basename(getcaller_bin_filepathname(spos))


def getcaller_bin_filepathname(spos=1):
    """File pathname of caller module."""
    _sf = stack()
    if spos >= len(_sf):
        return None
    module = getmodule(_sf[spos][0])  # mx = _sf[spos][0].f_locals['module']
    if not V3K:
        if module:
            return os.path.abspath(module.__file__)  # source
    else:
        # PEP 3147 -- PYC Repository Directories
        ret = find_loader(module.__name__)  #@UndefinedVariable
        if ret and hasattr(ret, 'path'):
            # check python native
            cdir = os.path.dirname(ret.path) + os.path.sep + '__pycache__'
            if os.path.exists(cdir):
                mname = re.sub(r"^.*[.]", '', ret.name)
                for xf in os.walk(cdir):
                    res = None
                    for xi in xf[2]:
                        if vers == re.sub(  #@UndefinedVariable
                                         mname + r".cpython-(3[0-9]).py[co]$", r'\1', xi):
                            res = cdir + os.path.sep + xi
                            if res[-1] == 'o':  # prio over '.pyc' on '.pyo'
                                break
                    return res
            else:
                return ret.path
        else:
            if ret:
                if hasattr(ret, 'name'):
                    fn = ret.get_filename(module.__name__)
                    fd = ret.get_data(fn)
                    return fn

            # FIXME: module.__name__ => 'baseOID'
            if module.__name__ in sys.builtin_module_names:
                return None
            else:
                bn = sys.modules[module.__name__].__file__
                if bn and os.path.exists(bn):
                    return bn

def getcaller_bin_pathname(spos=1):
    """pathname of caller source file."""
    if not V3K:
        return os.path.dirname(getcaller_bin_filepathname(spos + 1))
    else:
        _sf = stack()
        module = getmodule(_sf[spos][0])
        if hasattr(module, '__file__'):
            f = module.__file__
            return os.path.dirname(f)
        # PEP 3147 -- PYC Repository Directories


def getcaller_bin_pathname_rel(spos=1):
    """Relative pathname to first matching package directory of caller."""
    ax = os.path.normpath(getcaller_bin_filepathname(
        spos + 1))  # TODO: check for normpath
    for si in sys.path:
        si = os.path.normpath(si)
        if ax.startswith(si):
            return re.sub(si + r"[/\\\\]*(.+?)[/\\\\]*[^/\\\\]+.py[oc]*$", r"\1", ax)


def getcaller_bin_pathname_sub(spos=1):
    """sub-pathname to first matching module directory of caller."""
    ax = getcaller_bin_pathname_rel(spos + 1)
    return re.sub(r"[^/\\\\]*[/\\\\](.*)", r"\1", ax)


def getmodule_bin_filename(mod):
    """Basename of file for loaded module *mod*."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__') and os.path.exists(mod.__file__):
            return os.path.basename(mod.__file__)


def getmodule_bin_filepathname(mod):
    """File pathname of loaded module *mod*."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__') and os.path.exists(mod.__file__):
            return os.path.abspath(os.path.normpath(mod.__file__))


def getmodule_bin_pathname(mod):
    """Path name of loaded module."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__'):
            return os.path.normpath(os.path.dirname(mod.__file__))


def getmodule_bin_pathname_rel(mod, plist=None):
    """Relative path name to PYTHONPATH for loaded module."""
    if mod and ismodule(mod) and not isbuiltin(mod):
        if hasattr(mod, '__file__'):
            return __getpythonpath_rel(mod.__file__, plist)


def getmodule_bin_pathname_sub(mod, plist=None):
    """Path name for loaded module, relative to package path ."""
    ax = getmodule_bin_pathname_rel(mod, plist)
    if not V3K:
        return re.sub(r"[^/\\\\]*[/\\\\]", r"\1", ax)
    else:
        return re.sub(r"[^/\\\\]*[/\\\\]".encode('utf-8'), r"\1".encode('utf-8'), ax)



# -*- coding: utf-8 -*-
"""PySourceInfo - runtime type and source information on Python.

Based on the stack-frames of **inspect**, main parameter is **spos** 
- stack position. 

* *spos=1* is caller, *spos=2* is caller-of-caller, etc.. 

* 'r' is a regular expression for 'search()'.

Details see @local-manuals or @[https://pythonhosted.org/pysourceinfo/]
"""
from __future__ import absolute_import

import os
import sys
import re
from inspect import stack, getmodule, ismodule

from imp import is_builtin, is_frozen, find_module
from types import ModuleType, BuiltinFunctionType, BuiltinMethodType

from pysourceinfo import PySourceInfoError, V3K, \
    MT_UNKNOWN, MT_SOURCE, MT_COMPILED, MT_EXTENSION, MT_DIRECTORY, MT_BUILTIN, MT_FROZEN, \
    MT_COMPILED_OPT, MT_COMPILED_OPT1, MT_COMPILED_OPT2, MT_COMPILED_DEBUG

if V3K:
    from importlib import find_loader  # @UnresolvedImport


__author__ = 'Arno-Can Uestuensoez'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2010-2017 Arno-Can Uestuensoez" \
    " @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.21'
__uuid__ = '9de52399-7752-4633-9fdc-66c87a9200b8'

__docformat__ = "restructuredtext en"


def getcaller_module(spos=1):
    """Caller module."""
    _sf = stack()
    if spos >= len(_sf):
        return None
    return getmodule(_sf[spos][0])


def getcaller_module_name(spos=1):
    """Name of caller module, else *None*."""
    _sf = stack()
    if len(_sf) <= spos:
        return None
    module = getmodule(_sf[spos][0])
    if module:  # seems to be more accurate than getmodule_name
        if module.__name__ == '__main__':
            return os.path.basename(os.path.splitext(_sf[spos][1])[0])
        return module.__name__


def getcaller_module_name_sub(spos=1):
    """Name from package to module."""

    # TODO:
    _sf = stack()
    if len(_sf) <= spos:
        return None
    module = getmodule(_sf[spos][0])
    if module:
        return module.__name__  # seems to be more accurate than getmodule_name


def getcaller_module_oid(spos=1):
    """OID of caller module."""
    return getmodule_oid(getcaller_module(spos + 1))


def getcaller_module_oid_sub(spos=1):
    """OID of caller module."""
    return getmodule_oid_sub(getcaller_module_oid(spos))


def getcaller_name(spos=1):
    """Name of caller."""
    return stack()[spos][3]


def getcaller_package_name(spos=1):
    """Name of first matching package containing the caller.
    The package is defined as the the first part
    of the module name.
    """
    _sf = stack()
    if len(_sf) < spos:
        return None
    module = getmodule(_sf[spos][0])
    if module and module.__package__:
        return re.sub("^([^.]+).*", r'\1', module.__package__)
    else:
        return re.sub("^([^.]+).*", r'\1', getcaller_module_name(spos + 1))


def getmodule_by_id(i):
    """Loaded module by ID."""
    if i:
        return [x for x in sys.modules.values() if id(x) == i][0]


def getmodule_by_name(n):
    """Loaded module by given name."""
    if n and sys.modules.get(n):
        return sys.modules[n]


_gmn = re.compile(r'^.*[.]')


def getmodule_name(mod):
    """Name similar to basename of loaded module, same as getmodule_oid."""
    if mod and ismodule(mod) and hasattr(mod, '__name__'):
            return _gmn.sub('', mod.__name__)


def getmodule_name_sub(mod):
    """Name relative to PYTHONPATH/sys.path of loaded module, same as getmodule_oid_sub."""
    if mod and ismodule(mod) and hasattr(mod, '__name__'):
        return re.sub(r'^[^.]*[.]', r'', mod.__name__)


def getmodule_oid(mod):
    """Name relative to PYTHONPATH/sys.path of loaded module."""
    if mod and ismodule(mod) and hasattr(mod, '__name__'):
        return mod.__name__


def getmodule_oid_sub(mod):
    """Name relative to PYTHONPATH/sys.path of loaded module."""
    if mod and ismodule(mod):
        if hasattr(mod, '__name__'):
            return re.sub(r'^[^.]*[.]', r'', mod.__name__)


def getmodule_package_name(mod):
    """Name of package for loaded module."""
    mn = getmodule_name(mod)
    if mn:
        if hasattr(mod, '__name__'):
            return mod.__name__


def getmodule_type(mod):
    """Type for loaded module as defined by module 'imp.*'"""
    if not V3K:
        try:
            ret = find_module(getmodule_oid(mod))
            if ret and ret[2]:
                if type(ret[2]) is int:  # MT_SOURCE,MT_COMPILED,MT_EXTENSION
                    return ret[2]
                elif type(ret[2]) is tuple:  # MT_DIRECTORY,MT_BUILTIN
                    return ret[2][2]
                # TODO: PY_FROZEN
        except:
            if ismodule(mod) or type(mod) is ModuleType:
                try:
                    _n = getmodule_oid(mod)
                    if _n in sys.builtin_module_names or is_builtin(_n):
                        return MT_BUILTIN
                    else:
                        bn = sys.modules[_n].__file__
                        if bn:
                            bn = os.path.basename(bn)
                            if bn.startswith('__init__'):
                                return MT_DIRECTORY
                            elif bn[-2:] == 'py':
                                return MT_SOURCE
                            elif bn[-3:] in ('pyc',):
                                return MT_COMPILED_DEBUG
                            elif bn[-3:] in ('pyo'):
                                return MT_COMPILED_OPT  # either O1 or O2

                            elif bn[-3:] in ('so',):
                                return MT_EXTENSION
                            elif is_frozen(_n):
                                return MT_FROZEN
                except ImportError:
                    pass
                except:
                    raise
            return MT_UNKNOWN
    else:
        # PEP 3147 -- PYC Repository Directories
        ret = find_loader(getmodule_oid(mod))

        if ret and hasattr(ret, 'path'):
            # check python native
            cdir = os.path.dirname(ret.path) + os.path.sep + '__pycache__'
            if os.path.exists(cdir):
                v = '{0}{1}'.format(*sys.version_info[:2])
                mname = re.sub(r"^.*[.]", '', ret.name) + '.' + sys.implementation.cache_tag  # @UndefinedVariable
                for xf in os.walk(cdir):
                    for xi in xf[2]:
                        if xi.startswith(mname):
                            #
                            # for now supports: Python3, Pypy3
                            #
                            if xi[-3:]== "pyc":
                                return MT_COMPILED_DEBUG
                            elif xi[-3:]== "pyo":
                                if xi[-9:-4]== "opt-1":
                                    return MT_COMPILED_OPT1
                                if xi[-9:-4]== "opt-2":
                                    return MT_COMPILED_OPT2
                            else:
                                raise PySourceInfoError("Extension unknown:" + str(xi))

            else:
                return MT_SOURCE

        elif type(ret) in (BuiltinFunctionType, BuiltinMethodType,):
            return MT_BUILTIN

        else:
            _n = None
            if ret:

                if not hasattr(ret, 'name'):
                    if type(ret) == type:
                        # return MT_FROZEN
                        return MT_BUILTIN
                    else:
                        return MT_UNKNOWN
                else:
                    _n = ret.name
            else:
                try:
                    _n = getmodule_oid(mod)
                except ImportError:
                    pass
                except:
                    raise

            if _n in sys.builtin_module_names or is_builtin(_n):
                return MT_BUILTIN
            else:
                bn = sys.modules[_n].__file__
                if bn:
                    bn = os.path.basename(bn)
                    if bn.startswith('__init__'):
                        return MT_DIRECTORY
                    elif bn[-2:] == 'py':
                        return MT_SOURCE
                    elif bn[-3:] in ('pyc', 'pyo'):
                        return MT_COMPILED
                    elif bn[-3:] in ('so',):
                        return MT_EXTENSION
                    elif is_frozen(_n):
                        return MT_FROZEN

            return MT_UNKNOWN



"""Distribute 'pysourceinfo', a generic wrapper for 'inspect'.

   Installs 'pysourceinfo', adds/modifies the following helper features to standard
   'setuptools' options.

   Args:
      build_doc: Creates Sphinx based documentation with embeded javadoc-style
          API documentation, html only.

      build_sphinx: Creates documentation for runtime system by Sphinx, html only.
         Calls 'callDocSphinx.sh'.

      build_epydoc: Creates standalone documentation for runtime system by Epydoc, 
         html only.

      install_project_doc: Install a local copy into the doc directory of the project.

      instal_doc: Install a local copy of the previously build documents in 
          accordance to PEP-370.

      test: Runs PyUnit tests by discovery.

      usecases: Runs PyUnit UseCases by discovery, a lightweight
          set of unit tests.

      --sdk:
          Requires sphinx, epydoc, and dot-graphics.

      --no-install-required: Suppresses installation dependency checks, 
          requires appropriate PYTHONPATH.
      --offline: Sets online dependencies to offline, or ignores online
          dependencies.

      --exit: Exit 'setup.py'.

      --help-pysourceinfo: Displays this help.

   Returns:
      Results for success in installed 'pysourceinfo'.

   Raises:
      ffs.

"""
from __future__ import print_function

# priority is offline here - needs manual 'bootstrap', thus dropped ez_setup for now
# import ez_setup
# ez_setup.use_setuptools()

#
#*** common source header
#
__author__ = 'Arno-Can Uestuensoez'
__author_email__ = 'acue_sf2@sourceforge.net'
__license__ = "Artistic-License-2.0 + Forced-Fairplay-Constraints"
__copyright__ = "Copyright (C) 2015-2016 Arno-Can Uestuensoez @Ingenieurbuero Arno-Can Uestuensoez"
__version__ = '0.1.32'
__uuid__ = 'efed42d3-f801-4fbb-abfd-bd598d683a82'

_NAME = 'pysourceinfo'

# some debug
if __debug__:
    __DEVELTEST__ = True

#
#***
#
import os,sys
from setuptools import setup #, find_packages
import fnmatch
import re, shutil, tempfile

version = '{0}.{1}'.format(*sys.version_info[:2])
if  version <'3.5' and not version in ('2.7',):  # pragma: no cover
    raise Exception("Requires Python-2.7, or >=3.5")

#
# required for a lot for now, thus just do it
sys.path.insert(0,os.path.abspath(os.path.dirname(__file__)))

#
#*** ===>>> setup.py helper
#
def find_files(srcdir, *wildcards, **kw):
    """Assembles a list of package files for package_files.

        Args:
            srcdir: Source root.
            *wildcards: list of globs.
            **kw: Additional control of resolution:
                single_level: Flat only.
                subpath: Cut topmost path elemenr from listelements,
                    special for dictionaries.
                nopostfix: Drop filename postfix.
                packages: List packages only, else files.
                yield_folders:
        Returns:
            Results in an list.

        Raises:
            ffs.

    """
    def all_files(root, *patterns, **kw):
        ret=[]
        single_level = kw.get('single_level', False)
        subpath = kw.get('subpath', False)
        nopostfix = kw.get('nopostfix', True)
        packages = kw.get('packages', True)
        yield_folders = kw.get('yield_folders', True)

        for path, subdirs, files in os.walk(root):
            if yield_folders:
                files.extend(subdirs)
            files.sort( )

            if subpath:
                path=re.sub(r'^[^'+os.sep+']*'+os.sep, '',path)

            for name in files:
                if name in ('.gitignore', '.git', '.svn'):
                    continue

                for pattern in patterns:
                    if fnmatch.fnmatch(name, pattern):
                        if packages:
                            if not name == '__init__.py':
                                continue
                            ret.append(path)
                            continue
                        if nopostfix:
                            name=os.path.splitext(name)[0]

                        ret.append(os.path.join(path, name))

            if single_level:
                break
        return ret

    file_list = all_files(srcdir, *wildcards,**kw)
    return file_list

def usage():
    if __name__ == '__main__':
        import pydoc
        #FIXME: literally displayed '__main__'
        print(pydoc.help(__name__))
    else:
        help(str(os.path.basename(sys.argv[0]).split('.')[0]))

#
#* shortcuts
#

exit_code = 0

# custom doc creation by sphinx-apidoc
if 'build_sphinx' in sys.argv or 'build_doc' in sys.argv:
    try:
        os.makedirs('build'+os.sep+'apidoc'+os.sep+'sphinx')
    except:
        pass

    print("#---------------------------------------------------------")
    exit_code = os.system('./callDocSphinx.sh') # create apidoc
    print("#---------------------------------------------------------")
    print("Called/Finished callDocSphinx.sh => exit="+str(exit_code))
    if 'build_sphinx' in sys.argv:
        sys.argv.remove('build_sphinx')

# common locations
src0 = os.path.normpath("build/apidoc/sphinx/_build/html")
dst0 = os.path.normpath("build/apidoc/"+str(_NAME))    

# custom doc creation by sphinx-apidoc with embeded epydoc
if 'build_doc' in sys.argv:

    # copy sphinx to mixed doc
    if not os.path.exists(src0):
        raise Exception("Missing generated sphinx document source:"+str(src0))
    if os.path.exists(dst0):
        shutil.rmtree(dst0)
    shutil.copytree(src0, dst0)
    
    print("#---------------------------------------------------------")
    exit_code = os.system('epydoc --config docsrc/epydoc.conf') # create apidoc
    print("#---------------------------------------------------------")
    print("Called/Finished epydoc --config docsrc/epydoc.conf => exit="+str(exit_code))

    def _sed(filename, pattern, repl, flags=0):
        pattern_compiled = re.compile(pattern,flags)
        fname = os.path.normpath(filename)
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as ftmp:
            with open(fname) as src_file:
                for line in src_file:
                    ftmp.write(pattern_compiled.sub(repl, line))
    
        shutil.copystat(fname, ftmp.name)
        shutil.move(ftmp.name, fname)

    pt = '<a target="moduleFrame" href="toc-everything.html">Everything</a>'
    rp  = r'<a href="../index.html" target="_top">Home</a>'
    rp += r' - '
    rp += r'<a href="./index.html" target="_top">Top</a>'
    rp += r' - '
    rp += pt

    fn = dst0+'/epydoc/toc.html'
    _sed(fn, pt, rp, re.MULTILINE)  # @UndefinedVariable

    pt = '[@]local-manuals'
    rp  = r'[<a href="../index.html#table-of-contents" target="_top">@local-manuals</a>'
    #fn = dst0+'/epydoc/*.html'
    for flst in os.walk(dst0+'/epydoc/'):
        for fn in flst[2]:
            if fn[-5:] == '.html':
                _sed(flst[0]+os.path.sep+fn, pt, rp, re.MULTILINE)  # @UndefinedVariable
    
    pt = '[&][#]64[;]local-manuals'
    rp  = r'@[<a href="../index.html#table-of-contents" target="_top">local-manuals</a>]'
    for flst in os.walk(dst0+'/epydoc/'):
        for fn in flst[2]:
            if fn[-5:] == '.html':
                _sed(flst[0]+os.path.sep+fn, pt, rp, re.MULTILINE)  # @UndefinedVariable
    pt = re.compile(r'(<span class="codelink"><a href=.*pysourceinfo[.])([^\\-]+)([\\-][^#]*[#])([^"]+)(">source&nbsp;code</a></span>)')
    def rpfunc(match):
        rp  = r'[<span class="codelink"><a href="../'+match.group(2).lower()+'.html#'+match.group(4).lower()+'" target="_top">api</a></span>]&nbsp;'
        rp += match.group(1)+match.group(2)+match.group(3)+match.group(4)+match.group(5)
        return rp
    for flst in os.walk(dst0+'/epydoc/'):
        for fn in flst[2]:
            if fn[-5:] == '.html':
                _sed(flst[0]+os.path.sep+fn, pt, rpfunc)

    pt = '<h4>Next topic</h4>'
    rp  = r'<h4>API</h4><p class="topless"><a href="epydoc/index.html" title="API">Programming Interface</a></p>'
    rp += pt
    fn = dst0+'/index.html'
    _sed(fn, pt, rp, re.MULTILINE)  # @UndefinedVariable

    pt = r'<li><a class="reference internal" href="#table-of-contents">Table of Contents</a></li>'
    rp  = r'<li><a class="reference internal" href="shortcuts.html">Shortcuts</a></li>'
    rp += r'<li><a class="reference internal" href="howto.html">Howto</a></li>'
    rp += r'<li><a class="reference internal" href="install.html">Install</a></li>'
    rp += pt
    fn = dst0+'/index.html'
    _sed(fn, pt, rp, re.MULTILINE)  # @UndefinedVariable

    pt = '<h4>Previous topic</h4>'
    rp  = r'<h4>API</h4><p class="topless"><a href="epydoc/index.html" title="API">Programming Interface</a></p>'
    rp += pt
    patchlist = [
        'shortcuts.html',
        'pysourceinfo.html',
        'infolists.html',
        'fileinfo.html',
        'objectinfo.html',
        'bininfo.html',
        'helper.html',
    ]
    for px in patchlist:
        fn = dst0+os.sep+px
        _sed(fn, pt, rp, re.MULTILINE)  # @UndefinedVariable

    
    pt = '<h3>Quick search</h3>'
    rp  = r'<h4>API</h4><p class="topless"><a href="../../epydoc/index.html" title="API">Programming Interface</a></p>'
    rp += pt
    patchlist = [
        '_modules/pysourceinfo/fileinfo.html',
        '_modules/pysourceinfo/bininfo.html',
        '_modules/pysourceinfo/objectinfo.html',
        '_modules/pysourceinfo/helper.html',
        '_modules/pysourceinfo/infolists.html',
    ]
    for px in patchlist:
        fn = dst0+os.sep+px
        _sed(fn, pt, rp, re.MULTILINE)  # @UndefinedVariable

    sys.argv.remove('build_doc')

  

# custom doc creation by epydoc
if 'build_epydoc' in sys.argv:
    try:
        os.makedirs('build'+os.sep+'apidoc'+os.sep+'epydoc')
    except:
        pass
    
    print("#---------------------------------------------------------")
    exit_code = os.system('epydoc --config docsrc/epydoc-standalone.conf') # create apidoc
    print("#---------------------------------------------------------")
    print("Called/Finished epydoc --config docsrc/epydoc-standalone.conf => exit="+str(exit_code))
    sys.argv.remove('build_epydoc')

# install local project doc
if 'install_project_doc' in sys.argv:
    print("# install_project_doc.sh...")

    dstroot = os.path.normpath("doc/en/html/man3/")+os.sep
    
    try:
        os.makedirs(dstroot)
    except:
        pass

    if os.path.exists(dst0):
        if os.path.exists(dstroot+str(_NAME)):
            shutil.rmtree(dstroot+str(_NAME))
        shutil.copytree(dst0, dstroot+str(_NAME))


    src0 = os.path.normpath("build/apidoc/sphinx/_build/html")
    if os.path.exists(src0):
        if os.path.exists(dstroot+str(_NAME)+".sphinx"):
            shutil.rmtree(dstroot+str(_NAME)+".sphinx")
        shutil.copytree(src0, dstroot+str(_NAME)+".sphinx")

    src0 = os.path.normpath("build/apidoc/epydoc")
    if os.path.exists(src0):
        if os.path.exists(dstroot+str(_NAME)+".epydoc"):
            shutil.rmtree(dstroot+str(_NAME)+".epydoc")
        shutil.copytree(src0, dstroot+str(_NAME)+".epydoc")

    print("#")
    idx = 0
    for i in sys.argv: 
        if i == 'install_doc': break
        idx += 1
    
    print("#")
    print("Called/Finished PyUnit tests => exit="+str(exit_code))
    print("exit setup.py now: exit="+str(exit_code))
    sys.argv.remove('install_project_doc')

# install user doc
if 'install_doc' in sys.argv:
    print("# install_doc...")

    # set platform
    if sys.platform in ('win32'):
        dstroot = os.path.expandvars("%APPDATA%/Python/doc/en/html/man3/")
    else:
        dstroot = os.path.expanduser("~/.local/doc/en/html/man3/")
    dstroot = os.path.normpath(dstroot)+os.sep
    
    try:
        os.makedirs(dstroot)
    except:
        pass

    if os.path.exists(dst0):
        if os.path.exists(dstroot+str(_NAME)):
            shutil.rmtree(dstroot+str(_NAME))
        shutil.copytree(dst0, dstroot+str(_NAME))


    src0 = os.path.normpath("build/apidoc/sphinx/_build/html")
    if os.path.exists(src0):
        if os.path.exists(dstroot+str(_NAME)+".sphinx"):
            shutil.rmtree(dstroot+str(_NAME)+".sphinx")
        shutil.copytree(src0, dstroot+str(_NAME)+".sphinx")

    src0 = os.path.normpath("build/apidoc/epydoc")
    if os.path.exists(src0):
        if os.path.exists(dstroot+str(_NAME)+".epydoc"):
            shutil.rmtree(dstroot+str(_NAME)+".epydoc")
        shutil.copytree(src0, dstroot+str(_NAME)+".epydoc")

    print("#")
    idx = 0
    for i in sys.argv: 
        if i == 'install_doc': break
        idx += 1
    
    print("#")
    print("Called/Finished PyUnit tests => exit="+str(exit_code))
    print("exit setup.py now: exit="+str(exit_code))
    sys.argv.remove('install_doc')

version = '{0}.{1}'.format(*sys.version_info[:2])

# call of complete test suite by 'discover'
if 'tests' in sys.argv or 'test' in sys.argv:
    if os.path.dirname(__file__)+os.pathsep not in os.environ['PATH']:
        p0 = os.path.dirname(__file__)
        os.putenv('PATH', p0+os.pathsep+os.getenv('PATH',''))
        print("# putenv:PATH[0]="+str(p0))
    
    print("#")
    if version == '2.6': # pragma: no cover
        print("# Check 'inspect' paths - call in: tests")
        exit_code  = os.system('python -m discover -s tests -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: tests.30_libs")
        exit_code += os.system('python -m discover -s tests.30_libs -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: tests.30_libs.040_PySourceInfo")
        exit_code += os.system('python -m discover -s tests.30_libs.040_PySourceInfo -p CallCase.py') # traverse tree
    elif version == '2.7': # pragma: no cover
        print("# Check 'inspect' paths - call in: tests")
        exit_code  = os.system('python -m unittest discover -s tests -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: tests.30_libs")
        exit_code += os.system('python -m unittest discover -s tests.30_libs -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: tests.30_libs.040_PySourceInfo")
        exit_code += os.system('python -m unittest discover -s tests.30_libs.040_PySourceInfo -p CallCase.py') # traverse tree
    else:
        sys.stderr.write("ERROR:Version not supported:"+str(version))
        sys.exit(1)
    print("#")
    print("Called/Finished PyUnit tests => exit="+str(exit_code))
    print("exit setup.py now: exit="+str(exit_code))
    try:
        sys.argv.remove('test')
    except:
        pass
    try:
        sys.argv.remove('tests')
    except:
        pass
    

# call of complete UseCases by 'discover'
if 'usecases' in sys.argv or 'usecase' in sys.argv:
    if os.path.dirname(__file__)+os.pathsep not in os.environ['PATH']:
        p0 = os.path.dirname(__file__)
        os.putenv('PATH', p0+os.pathsep+os.getenv('PATH',''))
        print("# putenv:PATH[0]="+str(p0))
    
    print("#")
    if version == '2.6': # pragma: no cover
        print("# Check 'inspect' paths - call in: UseCases")
        exit_code = os.system('python -m discover -s UseCases -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: UseCases.PySourceInfo")
        exit_code += os.system('python -m discover -s UseCases.PySourceInfo -p CallCase.py') # traverse tree
    elif version == '2.7': # pragma: no cover
        print("# Check 'inspect' paths - call in: UseCases")
        exit_code = os.system('python -m unittest discover -s UseCases -p CallCase.py') # traverse tree
        print("# Check 'inspect' paths - call in: UseCases.PySourceInfo")
        exit_code += os.system('python -m unittest discover -s UseCases.PySourceInfo -p CallCase.py') # traverse tree
    else:
        sys.exit(1)
    print("#")
    print("Called/Finished PyUnit tests => exit="+str(exit_code))
    print("exit setup.py now: exit="+str(exit_code))
    try:
        sys.argv.remove('usecase')
    except:
        pass
    try:
        sys.argv.remove('usecases')
    except:
        pass

__sdk = False
if '--sdk' in sys.argv:
    __sdk = True
    sys.argv.remove('--sdk')

# Intentional HACK: ignore (online) dependencies, mainly foreseen for developement
__no_install_requires = False
if '--no-install-requires' in sys.argv:
    __no_install_requires = True
    sys.argv.remove('--no-install-requires')

# Intentional HACK: offline only, mainly foreseen for developement
__offline = False
if '--offline' in sys.argv:
    __offline = True
    __no_install_requires = True
    sys.argv.remove('--offline')

# Execution failed - Error.
if exit_code != 0:
    sys.exit(exit_code)

# Help on addons.
if '--help-pysourceinfo' in sys.argv:
    usage()
    sys.exit(0)

# Exit here.
if '--exit' in sys.argv:
    sys.exit(0)

# if pysourceinfo-specials only
if len(sys.argv)==1:
    sys.exit(exit_code)

#
#*** <<<=== setup.py helper
#


#
#*** setup.py configuration
#
_name= _NAME

_description = (
    "Utilities for simplified gain of runtime information on source code  and binary locations."
)

_README = os.path.join(os.path.dirname(__file__), 'README.md')
_long_description = open(_README).read()

_platforms='any'

_classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: Free To Use But Restricted",
    "License :: OSI Approved :: Artistic License",
    "Natural Language :: English",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: OS Independent",
    "Operating System :: POSIX :: BSD :: OpenBSD",
    "Operating System :: POSIX :: Linux",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: POSIX :: SunOS/Solaris",
    "Operating System :: POSIX",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",    
    "Programming Language :: Python :: 2.7",    
    "Programming Language :: Python :: 3",    
    "Programming Language :: Python :: 3.5",    
    "Programming Language :: Python :: 3.6",    
    "Programming Language :: Unix Shell",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities"
]

_keywords  = ' Python Syntax Packages Modules Files Linenumbers Filenames Modulenames Packagenames'
_keywords += ' RTTI PYTHONPATH inspect'

_packages = ["pysourceinfo"]
_scripts = []

_package_data = {
    'pysourceinfo': ['README.md','ArtisticLicense20.html', 'LICENSE', 'licenses-amendments.txt',
            'doc'
            ],
}

#_download_url="https://github.com/ArnoCan/pysourceinfo/"
_download_url="https://sourceforge.net/projects/pysourceinfo/files/"

_url='https://sourceforge.net/projects/pysourceinfo/'

_install_requires=[
#    'shutil',
#    'inspect',
#    're',
#    'termcolor'
]
if version == '2.6': # pragma: no cover
    _install_requires.extend(
        [
            'discover',
        ]
    )

if __sdk: # pragma: no cover
    _install_requires.extend(
        [
            'sphinx >= 1.4',
            'epydoc >= 3.0',
        ]
    )

_test_suite="tests.CallCase"

if __debug__:
    if __DEVELTEST__:
        print("#---------------------------------------------------------")
        print("packages="+str(_packages))
        print("#---------------------------------------------------------")
        print("package_data="+str(_package_data))
        print("#---------------------------------------------------------")


#
#*** ===>>> setup.py helper
#

# Intentional HACK: ignore (online) dependencies, mainly foreseen for developement
if __no_install_requires:
    print("#")
    print("# Changed to offline mode, ignore install dependencies completely.")
    print("# Requires appropriate PYTHONPATH.")
    print("# Ignored dependencies are:")
    print("#")
    for ir in _install_requires:
        print("#   "+str(ir))
    print("#")
    _install_requires=[]

#
#*** <<<=== setup.py helper
#


#
#*** do it now...
#
setup(name=_name,
      version=__version__,
      author=__author__,
      author_email=__author_email__,
      classifiers=_classifiers,
      description=_description,
      download_url=_download_url,
      install_requires=_install_requires,
      keywords=_keywords,
      license=__license__,
      long_description=_long_description,
      platforms=_platforms,
      url=_url,
      scripts=_scripts,
      packages=_packages,
      package_data=_package_data,
)

if '--help' in sys.argv:
    print()
    print("Help on usage extensions by "+str(_NAME))
    print("   --help-"+str(_NAME))
    print()
